#!/usr/bin/python3

from subprocess import call
import sys
import argparse
sys.path.append('/usr/lib/mintstick')
from mountutils import do_umount
import syslog

def execute(command):
    syslog.syslog(str(command))
    call(command)
    call(["sync"])

def raw_format(device_path, fstype, volume_label, uid, gid):

    do_umount(device_path)

    partition_path = "%s1" % device_path
    if fstype == "fat32":
        partition_type = "fat32"
    elif fstype == "exfat":
        partition_type = "ntfs"
    elif fstype == "ntfs":
        partition_type = "ntfs"
    elif fstype == "ext4":
        partition_type = "ext4"

    # First erase MBR and partition table , if any
    execute(["dd", "if=/dev/zero", "of=%s" % device_path, "bs=512", "count=1"])

    # Make the partition table
    execute(["parted", device_path, "mktable", "msdos"])

    # Make a partition (primary, with FS ID ext3, starting at 1MB & using 100% of space).
    # If it starts at 0% or 0MB, it's not aligned to MB's and complains
    execute(["parted", device_path, "mkpart", "primary", partition_type, "1", "100%"])

    # Call wipefs on the new partitions to avoid problems with old filesystem signatures
    execute(["wipefs", "-a", partition_path, "--force"])

    # Format the FS on the partition
    if fstype == "fat32":
        execute(["mkdosfs", "-F", "32", "-n", volume_label, partition_path])
    elif fstype == "exfat":
        execute(["mkfs.exfat", "-n", volume_label, partition_path])
    elif fstype == "ntfs":
        execute(["mkntfs", "-f", "-L", volume_label, partition_path])
    elif fstype == "ext4":
        execute(["mkfs.ext4", "-E", "root_owner=%s:%s" % (uid, gid), "-L", volume_label, partition_path])

    # Exit
    sys.exit(0)

def main():
    # parse command line options
    try:
        parser = argparse.ArgumentParser(description="Format USB",
                                         prog="mint-stick-format",
                                         epilog="Example : mint-stick-format -d /dev/sdj -f fat32 -l \"USB Stick\" -u 1000 -g 1000")
        parser.add_argument("-d", "--device", help="Device path", type=str, required=True)
        parser.add_argument("-f", "--filesystem", help="File system type", action="store",
                            type=str, choices=("fat32", "exfat", "ntfs", "ext4"), required=True)
        parser.add_argument("-u", "--uid", help="UID of the user", type=str, required=True)
        parser.add_argument("-g", "--gid", help="GID of the user", type=str, required=True)
        parser.add_argument("label", help="Volume label", type=str, nargs="*")
        args = parser.parse_args()
        print("Args", args)
        args.label = " ".join(args.label)
    except Exception as e:
        print(e)
        sys.exit(2)

    raw_format(args.device, args.filesystem, args.label, args.uid, args.gid)

if __name__ == "__main__":
    main()



#!/usr/bin/python3

import os, sys
import argparse
sys.path.append('/usr/lib/mintstick')
from mountutils import do_umount
import parted
import syslog

def raw_write(source, target):
    syslog.syslog(f"Writing '{source}' on '{target}'")
    try:
        do_umount(target)
        bs = 4096
        size=0
        input = open(source, 'rb')
        total_size = float(os.path.getsize(source))

        # Check if the ISO can fit ... :)
        device = parted.getDevice(target)
        device_size = device.getLength() * device.sectorSize
        if device_size < float(os.path.getsize(source)):
            input.close()
            print("nospace")
            exit(3)

        increment = total_size / 100

        written = 0
        output = open(target, 'wb')
        while True:
            buffer = input.read(bs)
            if len(buffer) == 0:
                break
            output.write(buffer)
            size = size + len(buffer)
            written = written + len(buffer)
            print(size/total_size)
            if (written >= increment):
                output.flush()
                os.fsync(output.fileno())
                written = 0

        output.flush()
        os.fsync(output.fileno())
        input.close()
        output.close()
        if size == total_size:
            print("1.0")
            exit (0)
        else:
            print("failed")
            exit (4)
    except Exception as e:
        syslog.syslog("An exception occured")
        syslog.syslog(str(e))
        print("failed")
        exit (4)

def main():
    # parse command line options
    try:
        parser = argparse.ArgumentParser(description="Format USB",
                                         prog="mint-stick-write",
                                         epilog="Example : mint-stick-write -s /foo/image.iso -t /dev/sdj")
        parser.add_argument("-s", "--source", help="Source iso path", type=str, required=True)
        parser.add_argument("-t", "--target", help="Target device path", type=str, required=True)
        args = parser.parse_args()
    except Exception as e:
        print(e)
        sys.exit(2)

    raw_write(args.source, args.target)

if __name__ == "__main__":
    main()



import sys
from subprocess import call
import syslog

def do_umount(target):
        mounts = get_mounted(target)
        if mounts:
            syslog.syslog(f"Unmounting all partitions of {target}.")
        for mount in mounts:
            device = mount[0]
            syslog.syslog(f"Trying to unmount {device}...")
            try:
                retcode = call(f"umount {device}", shell=True)
                if retcode < 0:
                    error = str(retcode)
                    syslog.syslog(f"Error, umount {device} was terminated by signal {error}")
                    sys.exit(6)
                else:
                    if retcode == 0:
                        syslog.syslog(f"{device} successfully unmounted")
                    else:
                        syslog.syslog(f"Error, umount {device} returned 0")
                        sys.exit(6)
            except OSError as e:
                error = str(e)
                syslog.syslog(f"Execution failed: {error}")
                sys.exit(6)


def get_mounted(target):
        try:
            lines = [line.strip("\n").split(" ") for line in open ("/etc/mtab", "r").readlines()]
            return [mount for mount in lines if mount[0].startswith(target)]
        except:
             syslog.syslog('Could not read mtab!')
             sys.exit(6)